import { Injectable } from '@angular/core';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { catchError } from 'rxjs/operators';
import { ErrorObservable } from 'rxjs/observable/ErrorObservable';

/**
 * Простой сервис для API с кешированием получаемых данных
 */

@Injectable()
export class CommonService {

    private apiUrl: string = '/bitrix/vuz/api/';
    private _cache: any = {};
    private _cacheDate: any = {};

    constructor(private http: HttpClient) {
    }

    public setUrl(url: string) {
        this.apiUrl = url;
    }

    clearCache(key?: string) {
        if (key) {
            if (this._cache.hasOwnProperty(key)) {
                delete this._cache[key];
                delete this._cacheDate[key];
            }
        } else {
            this._cache = {};
            this._cacheDate = {};
        }
    }

    public get(name: string): Observable<any> {
        if (!this._cache.hasOwnProperty(name)) {
            const url = this.apiUrl + name + '/';
            this._cacheDate[name] = new Date();

            this._cache[name] = this.http.get<any>(url).pipe(
                catchError(this.handleError)
            );
        }
        return this._cache[name];
    }

    public getById(name: string, id: number | string): Observable<any> {
        const key = name + '_' + id;
        if (!this._cache.hasOwnProperty(key)) {
            const url = this.apiUrl + name + '/' + id + '/';
            this._cacheDate[key] = new Date();

            this._cache[key] = this.http.get<any>(url).pipe(
                catchError(this.handleError)
            );
        }
        return this._cache[key];
    }

    public put(name: string, data: any, id?: number | string): Observable<any> {
        let url = '';
        if (id) {
            url = this.apiUrl + name + '/' + id + '/';
        } else {
            url = this.apiUrl + name + '/';
        }

        this.clearCache(name);

        return this.http.put<any>(url, data).pipe(
            catchError(this.handleError)
        );
    }

    public post(name: string, data: any): Observable<any> {
        const url = this.apiUrl + name + '/';

        this.clearCache(name);

        return this.http.post<any>(url, data).pipe(
            catchError(this.handleError)
        );
    }

    public delete(name: string, id: number | string): Observable<any> {
        const url = this.apiUrl + name + '/' + id + '/';

        this.clearCache(name + '_' + id);

        return this.http.delete<any>(url).pipe(
            catchError(this.handleError)
        );

    }

    private handleError(error: HttpErrorResponse) {
        if (error.error instanceof ErrorEvent) {
            // A client-side or network error occurred. Handle it accordingly.
            console.error('An error occurred:', error.error.message);
        } else {
            // The backend returned an unsuccessful response code.
            // The response body may contain clues as to what went wrong,
            console.error(error.error.error.message);
        }
        // return an ErrorObservable with a user-facing error message
        return new ErrorObservable(
            'Something bad happened; please try again later.');
    }
}
